package com.zanichelli.android.geotracker;

import android.Manifest;
import android.graphics.Color;
import android.location.Location;
import android.os.Bundle;

import com.google.android.gms.maps.*;
import com.google.android.gms.maps.model.*;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

// secondo le specifiche di Google, una classe che usa
// un fragment deve estendere almeno FragmentActivity.
// AppCompatActivity è una sua sottoclasse quindi è ok
// anche GeoTrackerActivity
public class MapsActivity extends GeoTrackerActivity
  implements OnMapReadyCallback,
  GoogleMap.OnMapClickListener,
  OnReverseGeocodingListener {

  private static final int REQUEST_START_LOCATION_NAME = 0;
  private static final int REQUEST_END_LOCATION_NAME = 1;

  private GoogleMap map;

  private Marker firstMarker = null;
  private Marker lastMarker = null;
  private Location firstLocation = null;
  private Location lastLocation = null;

  // indica se il percorso è stato già disegnato o meno
  private boolean isDrawn = false;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_maps);

    // ottiene il fragment definito nel file xml e avvia
    // la creazione della mappa
    SupportMapFragment mapFragment =
      (SupportMapFragment) getSupportFragmentManager()
        .findFragmentById(R.id.map);
    mapFragment.getMapAsync(this);
  }


  // metodo invocato una volta che la mappa è disponibile e pronta
  // per essere usata; da qui è possibile creare tutti i Marker
  // e le Polyline necessarie la prima volta
  @Override
  public void onMapReady(GoogleMap googleMap) {
    map = googleMap;

    // abilita la visualizzazione della posizione attuale
    if (this.checkPermission(
      Manifest.permission.ACCESS_FINE_LOCATION)
      && this.checkPermission(
      Manifest.permission.ACCESS_COARSE_LOCATION))
      map.setMyLocationEnabled(true);

    // imposta il tipo di mappa ibrida
    map.setMapType(GoogleMap.MAP_TYPE_HYBRID);
    // abilita l'intercettazione dell'interazione utente/mappa
    map.setOnMapClickListener(this);

    if (!isDrawn)
      drawJourney();

    isDrawn = true;
  }

  @Override
  public void onPermissionGranted(int requestCode) {
    super.onPermissionGranted(requestCode);
    try {
      map.setMyLocationEnabled(true);
    } catch (SecurityException exeption) {
    }
  }

  // metrodo che disegna il percorso aggiornato sulla mappa
  // se disponibile, altrimenti il disegno verrà effettuato
  // in seguito dal listener per lo stato della mappa
  @Override
  public void onStart() {
    super.onStart();
    if (map != null) {
      drawJourney();
      isDrawn = true;
    }
  }

  private void drawJourney() {
    // rimuove i disegni precedenti (Marker, Polyline, ...)
    map.clear();

    firstMarker = null;
    lastMarker = null;
    firstLocation = null;
    lastLocation = null;

    BufferedReader reader;
    String line;

    // opzioni di visualizzazione del tracciato
    PolylineOptions polylineOptions =
      new PolylineOptions();
    polylineOptions.color(Color.BLUE);
    polylineOptions.width(20);

    // marcatore di posizione
    Marker marker = null;
    // posizione cartesiana sulla mappa
    LatLng latLng = null;

    String[] token;
    Location location = null;

    // lettura dei dati da file
    try {
      reader = new BufferedReader(
        new FileReader(
          getApplicationContext().getFilesDir() + "/savedData.txt"));

      try {
        while ((line = reader.readLine()) != null) {

          token = line.split(",");
          location = new Location("");
          double lat = Double.parseDouble(token[1]);
          location.setLatitude(lat);
          double lon = Double.parseDouble(token[2]);
          location.setLongitude(lon);

          // conversione in formato LatLng
          latLng = new LatLng(location.getLatitude(),
            location.getLongitude());
          MarkerOptions markerOptions;

          if (marker == null) {
            // primo marker
            markerOptions = new MarkerOptions()
              .position(latLng)
              .title("Partenza: " + latLng.toString())
              .visible(true);

            firstMarker = marker = map.addMarker(markerOptions);
            firstLocation = location;
          } else {
            markerOptions = new MarkerOptions()
              .position(latLng)
              .title(latLng.toString())
              .visible(false);
            marker = map.addMarker(markerOptions);
          }

          polylineOptions.add(marker.getPosition());
        }

        // l’ultimo marker e’ visibile
        if (marker != null) {
          lastMarker = marker;
          lastLocation = location;

          if (marker == firstMarker) {
            // se l'ultimo marker coincide col primo
            marker.setTitle("Partenza/Arrivo: " + latLng.toString());
          } else {
            marker.setTitle("Arrivo: " + latLng.toString());
            marker.setVisible(true);
          }

          // impostazione della visualizzazione sull'ultima posizione
          map.moveCamera(CameraUpdateFactory.newLatLng(latLng));
          map.animateCamera(CameraUpdateFactory.zoomTo(15));

          // creazione del tracciato sulla mappa
          map.addPolyline(polylineOptions);

          // richieste asincrone di traduzione delle posizioni

          ReverseGeocoder.requestAddressFromLocation(
            REQUEST_START_LOCATION_NAME,
            firstLocation,
            this, this);

          if (firstMarker != lastMarker) {
            ReverseGeocoder.requestAddressFromLocation(
              REQUEST_END_LOCATION_NAME,
              lastLocation,
              this, this);
          }
        }
      } finally {
        reader.close();
      }
    } catch (IOException exeption) {

    }
  }

  // centratura della mappa su tocco dell’utente
  @Override
  public void onMapClick(LatLng point) {
    map.animateCamera(CameraUpdateFactory.newLatLng(point));
  }

  // metodo invocato quando la richiesta asincrona è terminata
  // con successo o meno (in quest'ultimo caso con risultato nullo)
  @Override
  public void onReverseGeocodingResult(
    int requestCode, String result) {

    // il fallimento della richiesta non richiede alcuna
    // operazione aggiuntiva, viene mantenuta la coppia
    // latitudine/longitudine usata in precedenza
    if (result == null)
      return;

    switch (requestCode) {
      case REQUEST_START_LOCATION_NAME:
        String prefix = (firstMarker != lastMarker) ?
          "Partenza: " : "Partenza/Arrivo: ";
        firstMarker.setTitle(prefix + result);
        break;
      case REQUEST_END_LOCATION_NAME:
        lastMarker.setTitle("Arrivo: " + result);
        break;

    }
  }
}
