package com.zanichelli.android.geotracker.position;

public class Position {

  public static final double PI = 3.141592653589793;
  public static final double WGS84_E2 = 0.006694379990197;
  public static final double WGS84_E4 = WGS84_E2 * WGS84_E2;
  public static final double WGS84_E6 = WGS84_E4 * WGS84_E2;
  public static final double WGS84_SEMI_MAJOR_AXIS = 6378137.0;
  public static final double WGS84_SEMI_MINOR_AXIS = 6356752.314245;
  public static final double UTM_LONGITUDE_OF_ORIGIN = 3.0 / 180.0 * PI;
  public static final double UTM_LATITUDE_OF_ORIGIN = 0.0;
  public static final double UTM_FALSE_EASTING = 500000.0;
  public static final double UTM_FALSE_NORTHING_N = 0.0;
  public static final double UTM_FALSE_NORTHING_S = 10000000.0;
  public static final double UTM_SCALE_FACTOR = 0.9996;

  private Date date;
  private Time time;
  private Latitude latitude;
  private Longitude longitude;

  private double m_calc(double latitude) {
    return (1.0 - WGS84_E2 / 4.0 - 3.0 * WGS84_E4 / 64.0 - 5.0 * WGS84_E6 / 256.0) * latitude - (3.0 * WGS84_E2 / 8.0 + 3.0 * WGS84_E4 / 32.0 + 45.0 * WGS84_E6 / 1024.0) * Math.sin(2.0 * latitude) + (15.0 * WGS84_E4 / 256.0 + 45.0 * WGS84_E6 / 1024.0) * Math.sin(4.0 * latitude) - (35.0 * WGS84_E6 / 3072.0) * Math.sin(6.0 * latitude);
  }


  public Position() {
    date = new Date(1, 1, 2000);
    time = new Time(0, 0, 0);
    latitude = new Latitude(0, 0, 0, Latitude.NORTH);
    longitude = new Longitude(0, 0, 0, Longitude.EAST);
  }

  public Position(Position p) {
    date = p.getDate();
    time = p.getTime();
    latitude = p.getLatitude();
    longitude = p.getLongitude();
  }

  public Position(Date d, Time t, Latitude lat, Longitude lon) {
    date = d;
    time = t;
    latitude = lat;
    longitude = lon;
  }

  public Date getDate() {
    return new Date(date);
  }

  public Time getTime() {
    return new Time(time);
  }

  public Latitude getLatitude() {
    return new Latitude(latitude);
  }

  public Longitude getLongitude() {
    return new Longitude(longitude);
  }

  public double getNorth() {
    double north;
    int int_zone;
    double M, M_origin, A, A2, e2_prim, C, T, v;
    double lat = latitude.getLatitude();
    double lon = longitude.getLongitude();
    int_zone = (int) (lon / 6.0);
    if (lon < 0) {
      int_zone = int_zone - 1;
    }
    lon -= (double) (int_zone) * 6.0;
    lon *= PI / 180.0;
    lat *= PI / 180.0;
    M = WGS84_SEMI_MAJOR_AXIS * m_calc(lat);
    M_origin = WGS84_SEMI_MAJOR_AXIS * m_calc(UTM_LATITUDE_OF_ORIGIN);
    A = (lon - UTM_LONGITUDE_OF_ORIGIN) * Math.cos(lat);
    A2 = A * A;
    e2_prim = WGS84_E2 / (1.0 - WGS84_E2);
    C = e2_prim * Math.pow(Math.cos(lat), 2.0);
    T = Math.tan(lat);
    T *= T;
    v = WGS84_SEMI_MAJOR_AXIS / Math.sqrt(1.0 - WGS84_E2 * Math.pow(Math.sin(lat), 2.0));
    north = UTM_SCALE_FACTOR * (M - M_origin + v * Math.tan(lat) * (A2 / 2.0 + (5.0 - T + 9.0 * C + 4.0 * C * C) * A2 * A2 / 24.0 + (61.0 - 58.0 * T + T * T + 600.0 * C - 330.0 * e2_prim) * A2 * A2 * A2 / 720.0));
    if (lat < 0) {
      north += UTM_FALSE_NORTHING_S;
    }
    return north;
  }

  public double getEast() {
    double east;
    int int_zone;
    double M, M_origin, A, A2, e2_prim, C, T, v;
    double lat = latitude.getLatitude();
    double lon = longitude.getLongitude();
    int_zone = (int) (lon / 6.0);
    if (lon < 0) {
      int_zone = int_zone - 1;
    }
    lon -= (double) (int_zone) * 6.0;
    lon *= PI / 180.0;
    lat *= PI / 180.0;
    M = WGS84_SEMI_MAJOR_AXIS * m_calc(lat);
    M_origin = WGS84_SEMI_MAJOR_AXIS * m_calc(UTM_LATITUDE_OF_ORIGIN);
    A = (lon - UTM_LONGITUDE_OF_ORIGIN) * Math.cos(lat);
    A2 = A * A;
    e2_prim = WGS84_E2 / (1.0 - WGS84_E2);
    C = e2_prim * Math.pow(Math.cos(lat), 2.0);
    T = Math.tan(lat);
    T *= T;
    v = WGS84_SEMI_MAJOR_AXIS / Math.sqrt(1.0 - WGS84_E2 * Math.pow(Math.sin(lat), 2.0));
    east = UTM_FALSE_EASTING + UTM_SCALE_FACTOR * v * (A + (1.0 - T + C) * A2 * A / 6.0 + (5.0 - 18.0 * T + T * T + 72.0 * C - 58.0 * e2_prim) * A2 * A2 * A / 120.0);
    return east;
  }

  // Calcola la distanza spaziale
  // in metri da una posizione
  public double distance(Position p) {
    return Math.sqrt(Math.pow(getNorth() - p.getNorth(), 2) + Math.pow(getEast() - p.getEast(), 2));
  }

  // calcola la differenza temporale
  // in secondi da una posizione
  public int elapsedTime(Position p) {
    double hours;
    int seconds;

    hours = date.dayDifference(p.getDate()) * 24;
    hours += time.hourDifference(p.getTime());
    seconds = (int) (hours * 3600.);
    return seconds;
  }
}